/**
 * Copyright 2012 OW2 Shelbie
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.shelbie.core.internal;

import org.apache.felix.ipojo.annotations.Component;
import org.apache.felix.ipojo.annotations.Provides;
import org.apache.felix.service.command.CommandSession;
import org.codehaus.plexus.interpolation.InterpolationException;
import org.codehaus.plexus.interpolation.PropertiesBasedValueSource;
import org.codehaus.plexus.interpolation.RegexBasedInterpolator;
import org.fusesource.jansi.Ansi;
import org.ow2.shelbie.core.IPromptProvider;
import org.ow2.shelbie.core.console.Constants;
import org.ow2.shelbie.core.console.SessionValueSource;

/**
 * @author Loris Bouzonnet
 */
@Component
@Provides
public class DefaultPromptProvider implements IPromptProvider {

    /**
     * Default prompt.
     * TODO Use Ansi coloring
     */
    private static final String DEFAULT_PROMPT = "${user.name}@${application.name}$ ";

    public String getPrompt(final CommandSession session) {

        String prompt = (String) session.get(Constants.PROMPT_VARIABLE);
        if (prompt == null) {
            prompt = DEFAULT_PROMPT;
        }

        // TODO Need some caching
        // Interpolate prompt
        RegexBasedInterpolator interpolator = new RegexBasedInterpolator();
        // Allow resolution of System properties
        interpolator.addValueSource( new PropertiesBasedValueSource(System.getProperties()));
        // then add values from the CommandSession
        interpolator.addValueSource(new SessionValueSource(session));

        // TODO Do we have some other sources of values that could be evaluated ?

        // Resolve the prompt
        try {
            return interpolator.interpolate(prompt);
        } catch (InterpolationException e) {
            printDebug(session, "Cannot interpolate prompt '" + prompt + "'. Error: " + e.getMessage());
            return DEFAULT_PROMPT;
        }
    }

    private void printDebug(final CommandSession session, final String message) {
        String error = Ansi.ansi().fg(Ansi.Color.GREEN)
                .a("Debug: ")
                .reset().toString();
        session.getConsole().println(error + message);
    }

}
