/**
 * Copyright 2012 OW2 Shelbie
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.shelbie.core.internal.handler;

import org.apache.felix.gogo.commands.Action;
import org.apache.felix.gogo.commands.basic.ActionPreparator;
import org.apache.felix.ipojo.InstanceManager;

/**
 * A Stateful Command is a command whose Action keeps state in the instance.
 * Just like EJB stateful beans, there is only one instance bound to a client.
 * That means that if the user perform multiple invocation of the same command,
 * the same instance will be used.
 */
public class StatefulGogoCommand extends GogoCommand {

    private Action unique;

	public StatefulGogoCommand(InstanceManager manager,
                               ActionPreparator preparator) {
		super(manager, preparator);
	}

    @Override
	public Action createNewAction() {
        if (unique == null) {
            unique = (Action) manager.createPojoObject();
        }
        return unique;
	}

    @Override
    public void release() {
        if (unique != null) {
            manager.deletePojoObject(unique);
        }
        this.unique = null;
        super.release();
    }

}
